from xhs_spider.one import OneNote
from xhs_spider.profile import Profile
from xhs_spider.xhs_utils.xhs_util import *
class Home:
    def __init__(self):
        self.more_url = 'https://edith.xiaohongshu.com/api/sns/web/v1/user_posted'
        self.profile = Profile()
        self.oneNote = OneNote(self.profile.cookies)
        self.headers = get_headers()
        self.params = get_params()
        self.cookies = self.profile.cookies

    # 主页
    def get_all_note_info(self, url):
        profile = self.profile.get_profile_info(url)
        note_id_list = []
        user_id = profile.userId
        cursor = ''
        self.params['user_id'] = user_id
        self.params['cursor'] = cursor
        while True:
            api = f"/api/sns/web/v1/user_posted?num=30&cursor={cursor}&user_id={user_id}&image_scenes="
            ret = js.call('get_xs', api, '', self.cookies['a1'])
            self.headers['x-s'], self.headers['x-t'] = ret['X-s'], str(ret['X-t'])
            response = requests.get(self.more_url, headers=self.headers, cookies=self.cookies, params=self.params)
            res = response.json()
            data = res["data"]
            if not data["notes"][0]:
                print(f"用户{user_id}没有笔记")
                break
            cursor, has_more, note_list = data["cursor"], data["has_more"], data["notes"]
            self.params['cursor'] = cursor
            for note in note_list:
                note_id_list.append(note['note_id'])
            if not has_more:
                break
        return note_id_list, profile

    # 主页
    def save_all_note_info(self, url, need_cover=False):
        profile = self.profile.save_profile_info(url)
        user_id = profile.userId
        cursor = ''
        self.params['user_id'] = user_id
        self.params['cursor'] = cursor
        index = 0
        while True:
            api = f"/api/sns/web/v1/user_posted?num=30&cursor={cursor}&user_id={user_id}&image_scenes="
            ret = js.call('get_xs', api, '', self.cookies['a1'])
            self.headers['x-s'], self.headers['x-t'] = ret['X-s'], str(ret['X-t'])
            response = requests.get(self.more_url, headers=self.headers, cookies=self.cookies, params=self.params)
            res = response.json()
            data = res["data"]
            if not data["notes"][0]:
                print(f"用户{user_id}没有笔记")
                break
            cursor, has_more, note_list = data["cursor"], data["has_more"], data["notes"]
            self.params['cursor'] = cursor
            for note in note_list:
                index += 1
                info = f'第{index}个笔记, '
                self.oneNote.save_one_note_info(self.oneNote.detail_url + note['note_id'], need_cover, info)
            if not has_more:
                break
        print(f'用户 {profile.nickname} 全部视频信息保存成功')


    def main(self, url_list):
        # url_list = [
        #     'https://www.xiaohongshu.com/user/profile/6185ce66000000001000705b',
        #     'https://www.xiaohongshu.com/user/profile/6034d6f20000000001006fbb',
        # ]
        for url in url_list:
            try:
                self.save_all_note_info(url)
            except:
                print(f'用户 {url} 查询失败')


if __name__ == '__main__':
    home = Home()
    url_list = [
        'https://www.xiaohongshu.com/user/profile/6185ce66000000001000705b',
        'https://www.xiaohongshu.com/user/profile/6034d6f20000000001006fbb',
    ]
    home.main(url_list)




import json
from xhs_spider.xhs_utils.xhs_util import *

class OneNote:
    def __init__(self, cookies=None):
        self.feed_url = 'https://edith.xiaohongshu.com/api/sns/web/v1/feed'
        self.detail_url = 'https://www.xiaohongshu.com/explore/'
        self.headers = get_headers()
        self.params = get_params()
        if cookies is None:
            self.cookies = check_cookies()
        else:
            self.cookies = cookies
    # 单个视频
    def get_one_note_info(self, url):
        note_id = url.split('/')[-1]
        data = get_note_data(note_id)
        data = json.dumps(data, separators=(',', ':'))
        ret = js.call('get_xs', '/api/sns/web/v1/feed', data, self.cookies['a1'])
        self.headers['x-s'], self.headers['x-t'] = ret['X-s'], str(ret['X-t'])
        response = requests.post(self.feed_url, headers=self.headers, cookies=self.cookies, data=data)
        res = response.json()
        try:
            data = res['data']['items'][0]
        except:
            print(f'笔记 {note_id} 不允许查看')
            return
        note = handle_note_info(data)
        return note

    # cover 是否覆盖
    def save_one_note_info(self, url, need_cover=False, info='', dir_path='datas'):
        try:
            note = self.get_one_note_info(url)
            nickname = norm_str(note.nickname)
            user_id = note.user_id
            title = norm_str(note.title)
            if title.strip() == '':
                title = f'无标题'
            path = f'./{dir_path}/{nickname}_{user_id}/{title}_{note.note_id}'
            exist = check_and_create_path(path)
            if exist and not need_cover:
                print(f'用户: {nickname}, 标题: {title} 本地已存在，跳过保存')
                return note
            save_note_detail(path, note)
            note_type = note.note_type
            if note_type == 'normal':
                for img_index, img in enumerate(note.image_list):
                    img_url = img['info_list'][1]['url']
                    download_media(path, f'image_{img_index}', img_url, 'image', f'第{img_index}张图片')

            elif note_type == 'video':
                img_url = note.image_list[0]['info_list'][1]['url']
                download_media(path, 'cover', img_url, 'image', '视频封面')
                video_url = note.video_addr
                download_media(path, 'video', video_url, 'video')
            print(f'用户: {nickname}, {info}标题: {title} 笔记保存成功')
            print('===================================================================')
            return note
        except:
            print(f'笔记 {url} 保存失败')
            return

    def main(self, url_list):
        # url_list = [
        #     'https://www.xiaohongshu.com/explore/64356527000000001303282b',
        #     # 'https://www.xiaohongshu.com/explore/63d625f8000000001d01042c',
        #     # 'https://www.xiaohongshu.com/explore/61ac8820000000002103a8aa',
        #     # 'https://www.xiaohongshu.com/explore/62d2699c000000001200f101',
        #     # 'https://www.xiaohongshu.com/explore/637f0938000000001f012d15',
        # ]
        for url in url_list:
            try:
                self.save_one_note_info(url)
            except:
                print(f'笔记 {url} 保存失败')
                continue

if __name__ == '__main__':
    one_note = OneNote()
    url_list = [
        'https://www.xiaohongshu.com/explore/64356527000000001303282b',
        # 'https://www.xiaohongshu.com/explore/63d625f8000000001d01042c',
        # 'https://www.xiaohongshu.com/explore/61ac8820000000002103a8aa',
        # 'https://www.xiaohongshu.com/explore/62d2699c000000001200f101',
        # 'https://www.xiaohongshu.com/explore/637f0938000000001f012d15',
    ]
    one_note.main(url_list)


class User_Detail():
    def __init__(self, id, userId, nickname, avatar, desc, follows, fans, interaction, ipLocation, gender, tags):
        self.id = id
        self.userId = userId
        self.nickname = nickname
        self.avatar = avatar
        self.desc = desc
        self.follows = follows
        self.fans = fans
        self.interaction = interaction
        self.ipLocation = ipLocation
        self.gender = gender
        self.tags = tags


    def __str__(self):
        # 每个值都要换行
        return f'id: {self.id}\n' \
                f'userId: {self.userId}\n' \
                f'nickname: {self.nickname}\n' \
                f'avatar: {self.avatar}\n' \
                f'desc: {self.desc}\n' \
                f'follows: {self.follows}\n' \
                f'fans: {self.fans}\n' \
                f'interaction: {self.interaction}\n' \
                f'ipLocation: {self.ipLocation}\n' \
                f'gender: {self.gender}\n' \
                f'tags: {self.tags}\n' \






import os
import re
import sys
import time
import execjs
import requests
from xhs_spider.pojo.note import Note_Detail
from xhs_spider.pojo.user import User_Detail

current_directory = os.path.dirname(os.path.abspath(__file__))
file_path = os.path.abspath(os.path.join(current_directory, '../static', 'info.js'))
js = execjs.compile(open(file_path, 'r', encoding='utf-8').read())
def decodedUniChars(url):
    decodedUniChars = url.encode('utf-8').decode("unicode_escape")
    return decodedUniChars

def norm_str(str):
    new_str = re.sub(r"|[\\/:*?\"<>| ]+", "", str).replace('\n', '').replace('\r', '')
    return new_str

# 时间戳1681220903000 转YYYY-MM-DD HH:MM:SS
def timestamp_to_str(timestamp):
    time_local = time.localtime(timestamp / 1000)
    dt = time.strftime("%Y-%m-%d %H:%M:%S", time_local)
    return dt

def check_and_create_path(path):
    if not os.path.exists(path):
        os.makedirs(path)
        return False
    return True

def check_path(path):
    if not os.path.exists(path):
        return False
    return True

def timestamp_to_time(timestamp):
    time_local = time.localtime(timestamp / 1000)
    dt = time.strftime("%Y年%m月%d日%H点%M分%S秒", time_local)
    return dt
def download_media(path, name, url, type, info=''):
    # 5次错误机会
    for i in range(5):
        try:
            if type == 'image':
                # print(f"{info}图片开始下载, {url}")
                print(f"{info}图片开始下载")
                content = requests.get(url).content
                with open(path + '/' + name + '.jpg', mode="wb") as f:
                    f.write(content)
                    print(f"{info}图片下载完成")
            elif type == 'video':
                print(f"{name}开始下载, {url}")
                start_time = time.time()
                res = requests.get(url, stream=True)
                size = 0
                chunk_size = 1024 * 1024
                content_size = int(res.headers["content-length"])
                with open(path + '/' + name + '.mp4', mode="wb") as f:
                    for data in res.iter_content(chunk_size=chunk_size):
                        f.write(data)
                        size += len(data)
                        percentage = size / content_size
                        print(f'\r视频:%.2fMB\t' % (content_size / 1024 / 1024),
                              '下载进度:[%-50s%.2f%%]耗时: %.1fs, ' % ('>' * int(50 * percentage), percentage * 100, time.time() - start_time),
                              end='')
                    print(f"{name}下载完成")
            break
        except:
            print(f"第{i+1}次下载失败，重新下载, 剩余{4-i}次机会")
            continue

def handle_profile_info(userId, html_text):
    true, false, null, undefined = True, False, None, None
    info = re.findall(r'<script>window.__INITIAL_STATE__=(.*?)</script>', html_text)[0]
    info = eval(info)
    nickname = info['user']['userPageData']['basicInfo']['nickname']
    avatar = info['user']['userPageData']['basicInfo']['images']
    avatar = decodedUniChars(avatar)
    desc = info['user']['userPageData']['basicInfo']['desc']
    follows = info['user']['userPageData']['interactions'][0]['count']
    fans = info['user']['userPageData']['interactions'][1]['count']
    interaction = info['user']['userPageData']['interactions'][2]['count']
    ipLocation = info['user']['userPageData']['basicInfo']['ipLocation']
    gender = info['user']['userPageData']['basicInfo']['gender']
    if gender == 0:
        gender = '男'
    elif gender == 1:
        gender = '女'
    else:
        gender = '未知'
    tags_temp = info['user']['userPageData']['tags']
    tags = []
    for tag in tags_temp:
        try:
            tags.append(tag['name'])
        except:
            pass
    user_detail = User_Detail(None, userId, nickname, avatar, desc, follows, fans, interaction, ipLocation, gender, tags)
    return user_detail

def save_user_detail(path, user):
    with open(f'{path}/detail.txt', mode="w", encoding="utf-8") as f:
        # 逐行输出到txt里
        f.write(f"主页url: {f'https://www.douyin.com/user/{user.userId}'}\n")
        f.write(f"用户名: {user.nickname}\n")
        f.write(f"介绍: {user.desc}\n")
        f.write(f"关注数量: {user.follows}\n")
        f.write(f"粉丝数量: {user.fans}\n")
        f.write(f"作品被赞和收藏数量: {user.interaction}\n")
        f.write(f"ip地址: {user.ipLocation}\n")
        f.write(f'性别: {user.gender}\n')
        f.write(f'标签: {user.tags}\n')

def save_note_detail(path, note):
    with open(path + '/' + 'detail.txt', mode="w", encoding="utf-8") as f:
        # 逐行输出到txt里
        f.write(f"笔记url: {f'https://www.xiaohongshu.com/explore/{note.note_id}'}\n")
        f.write(f'笔记类型: {note.note_type}\n')
        f.write(f"笔记标题: {note.title}\n")
        f.write(f"笔记描述: {note.desc}\n")
        f.write(f"笔记点赞数量: {note.liked_count}\n")
        f.write(f"笔记收藏数量: {note.collected_count}\n")
        f.write(f"笔记评论数量: {note.comment_count}\n")
        f.write(f"笔记分享数量: {note.share_count}\n")
        f.write(f"笔记上传时间: {timestamp_to_str(note.upload_time)}\n")
        f.write(f"笔记标签: {note.tag_list}\n")
        f.write(f"笔记ip归属地: {note.ip_location}\n")

def handle_note_info(data):
    note_id = data['id']
    note_type = data['note_card']['type']
    user_id = data['note_card']['user']['user_id']
    nickname = data['note_card']['user']['nickname']
    avatar = data['note_card']['user']['avatar']
    title = data['note_card']['title']
    desc = data['note_card']['desc']
    liked_count = data['note_card']['interact_info']['liked_count']
    collected_count = data['note_card']['interact_info']['collected_count']
    comment_count = data['note_card']['interact_info']['comment_count']
    share_count = data['note_card']['interact_info']['share_count']
    if note_type == 'video':
        video_addr = 'https://sns-video-bd.xhscdn.com/' + data['note_card']['video']['consumer']['origin_video_key']
    else:
        video_addr = ''
    image_list = data['note_card']['image_list']
    tags_temp = data['note_card']['tag_list']
    tags = []
    for tag in tags_temp:
        try:
            tags.append(tag['name'])
        except:
            pass
    upload_time = data['note_card']['time']
    if 'ip_location' in data['note_card']:
        ip_location = data['note_card']['ip_location']
    else:
        ip_location = '未知'
    note_detail = Note_Detail(None, note_id, note_type, user_id, nickname, avatar, title, desc, liked_count, collected_count, comment_count, share_count, video_addr, image_list, tags, upload_time, ip_location)
    return note_detail

def get_cookies():
    return {
        "xsecappid": "",
        "a1": "",
        "webId": "",
        "gid": "",
        "webBuild": "3.3.4",
        "web_session": "",
        "websectiga": "",
        "sec_poison_id": ""
    }

def get_home_headers():
    return {
        "authority": "www.xiaohongshu.com",
        "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "sec-ch-ua": "\"Microsoft Edge\";v=\"117\", \"Not;A=Brand\";v=\"8\", \"Chromium\";v=\"117\"",
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": "\"Windows\"",
        "sec-fetch-dest": "document",
        "sec-fetch-mode": "navigate",
        "sec-fetch-site": "none",
        "sec-fetch-user": "?1",
        "upgrade-insecure-requests": "1",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36 Edg/117.0.2045.47"
    }
def get_headers():
    return {
        "authority": "edith.xiaohongshu.com",
        "accept": "application/json, text/plain, */*",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
        "content-type": "application/json;charset=UTF-8",
        "origin": "https://www.xiaohongshu.com",
        "referer": "https://www.xiaohongshu.com/",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36 Edg/115.0.1901.188",
        "x-s": "",
        "x-t": ""
    }

def get_note_data(note_id):
    return {
        "source_note_id": note_id,
        "image_scenes": [
            "CRD_PRV_WEBP",
            "CRD_WM_WEBP"
        ]
    }
def get_search_data():
    return {
        "image_scenes": "FD_PRV_WEBP,FD_WM_WEBP",
        "keyword": "",
        "note_type": "0",
        "page": "",
        "page_size": "20",
        "search_id": "2c7hu5b3kzoivkh848hp0",
        "sort": "general"
    }
def get_params():
    return {
        "num": "30",
        "cursor": "",
        "user_id": "",
        "image_scenes": ""
    }

def check_cookies():
    more_url = 'https://edith.xiaohongshu.com/api/sns/web/v1/user_posted'
    params = get_params()
    headers = get_headers()
    print(f'github: https://github.com/cv-cat/Spider_XHS')
    try:
        if not os.path.exists("./cookies.txt"):
            with open("./cookies.txt", "w", encoding="utf-8") as f:
                file_path = os.path.abspath(os.path.join(current_directory, '../static', 'cookies.txt'))
                cookies_template = open(file_path, 'r', encoding='utf-8').read()
                f.write(cookies_template)
            raise Exception("初次运行, 生成cookies.txt, 请手动更改cookies.txt文件")
        test_user_id = '5ad2ede14eacab146f865fe9'
        with open("./cookies.txt", "r", encoding="utf-8") as f:
            cookies_obj = f.read()
        cookies_local = eval(cookies_obj)
        params['user_id'] = test_user_id
        params['cursor'] = ''
        api = f"/api/sns/web/v1/user_posted?num=30&cursor=&user_id={test_user_id}&image_scenes="
        a1 = cookies_local['a1']
        try:
            ret = js.call('get_xs', api, '', a1)
        except:
            print('缺少nodejs环境')
            return
        headers['x-s'], headers['x-t'] = ret['X-s'], str(ret['X-t'])
        response = requests.get(more_url, headers=headers, cookies=cookies_local, params=params)
        res = response.json()
        if not res["success"]:
            raise Exception("cookie失效")
        else:
            print("cookie有效")
            return cookies_local
    except Exception as e:
        print(f"cookie失效，请手动更改cookies.txt文件, {e}")
        sys.exit(1)
